#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define NMAX 10000000
#define PI25DT 3.141592653589793238462643

int main(int argc, char **argv)
{
    int rank, size, i, n;
    long int count = 0;
    double x1, x2, w, mypi, pi, t0, t1, mywctime, wctime;

    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    MPI_Comm_size(MPI_COMM_WORLD, &size);

    srand((rank+1)*size);
    n = NMAX/size;
    t0 = MPI_Wtime();
    for (i = 0; i < n; ++i) {
        x1 = ((double)rand()) / (RAND_MAX + 1.0);
        x2 = ((double)rand()) / (RAND_MAX + 1.0);
        w = x1 * x1 + x2 * x2;
        if (w <= 1.0)   count += 1;
    }
    mypi = (4.0 * count) / NMAX;
    MPI_Reduce(&mypi, &pi, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD); 

    t1 = MPI_Wtime();    mywctime = t1 - t0;
    MPI_Reduce(&mywctime, &wctime, 1, MPI_DOUBLE, MPI_MAX, 0, MPI_COMM_WORLD); 
    if (rank == 0)
        printf("pi is approximately %.16f, Error is %.16f, wctime is %e seconds\n",
               pi, fabs(pi - PI25DT), wctime);

    MPI_Finalize();
    return 0;
}

