# Calculating PI using random numbers (Monte Carlo Method)
import math
import random
from mpi4py import MPI

rank = MPI.COMM_WORLD.Get_rank()
size = MPI.COMM_WORLD.Get_size()

count = 0
NMAX = 10000000 # number of iterations (greater the better)
n = NMAX / size
tic = MPI.Wtime()
for i in xrange(n):
    x1 = random.random()
    x2 = random.random()
    if (x1 * x1 + x2 * x2) <= 1.0: count += 1

mypi = MPI.COMM_WORLD.gather((4.0 * count / NMAX), root=0)      # seria mais simples usar um Reduce
if rank == 0: pi = sum(mypi)                                    # mas pode ser feito assim tambem...

toc = MPI.Wtime();  wct = MPI.COMM_WORLD.gather(toc-tic, root=0)
if rank == 0:
    for task, time in enumerate(wct):
        print('wall clock time: {0:e} seconds (task {1:d})'.format(time, task))
    def mean(seq): return sum(seq)/len(seq)
    print('all tasks, mean: {0:e} seconds'.format(mean(wct)))
    print('all tasks, min:  {0:e} seconds'.format(min(wct)))
    print('all tasks, max:  {0:e} seconds'.format(max(wct)))
    print('all tasks, sum:  {0:e} seconds'.format(sum(wct)))
    print "pi is approximately {0:.16f}, Error is {1:.16f}".format(pi, abs(pi - math.pi))

